import { useSelector } from "react-redux";
import { AppState } from "@/state/reducer";
import { getSubmissionStatus, SubmissionState, SubmissionStatus } from "@/state/reducer/submission";
import { useTranslation } from "react-i18next";
import { Box, Button, ButtonProps, StepProps } from "@material-ui/core";
import { FileDownloadOutline, FileUploadOutline } from "mdi-material-ui/index";
import { route } from "@/routing";
import { Link as RouterLink } from "react-router-dom";
import { Actions, Step } from "@/components";
import React, { HTMLProps } from "react";
import { Alert, AlertTitle } from "@material-ui/lab";
import { Deadlines, Edition, getEditionDeadlines } from "@/data/edition";
import { ContactAction, Status } from "@/pages/steps/common";
import { Description as DescriptionIcon } from "@material-ui/icons";

const PlanActions = () => {
    const status = useSelector<AppState, SubmissionStatus>(state => getSubmissionStatus(state.plan));
    const { t } = useTranslation();

    const ReviewAction = (props: ButtonProps) =>
        <Button startIcon={ <FileDownloadOutline /> } color="primary" { ...props }>{ t('steps.plan.download') }</Button>

    const FormAction = ({ children = t('steps.plan.form'), ...props }: ButtonProps) =>
        <Button to={ route("internship_plan") } variant="contained" color="primary" component={ RouterLink } startIcon={ <FileUploadOutline /> }>
            { t('steps.plan.submit') }
        </Button>

    const TemplateAction = (props: ButtonProps) =>
        <Button href="https://eti.pg.edu.pl/documents/611675/100028367/indywidualny%20program%20praktyk" startIcon={ <DescriptionIcon/> } { ...props }>
            { t('steps.plan.template') }
        </Button>

    switch (status) {
        case "awaiting":
            return <Actions>
                <ReviewAction />
            </Actions>
        case "accepted":
            return <Actions>
                <ReviewAction/>
                <FormAction variant="outlined" color="secondary">{ t('send-again') }</FormAction>
            </Actions>
        case "declined":
            return <Actions>
                <FormAction>{ t('fix-errors') }</FormAction>
                <ReviewAction />
                <TemplateAction />
                <ContactAction/>
            </Actions>
        case "draft":
            return <Actions>
                <FormAction />
                <TemplateAction />
            </Actions>

        default:
            return <Actions/>
    }
}

export const PlanComment = (props: HTMLProps<HTMLDivElement>) => {
    const { comment, declined } = useSelector<AppState, SubmissionState>(state => state.plan);
    const { t } = useTranslation();

    return comment ? <Alert severity={ declined ? "error" : "warning" } { ...props as any }>
        <AlertTitle>{ t('comments') }</AlertTitle>
        { comment }
    </Alert> : null
}

export const PlanStep = (props: StepProps) => {
    const { t } = useTranslation();

    const submission = useSelector<AppState, SubmissionState>(state => state.plan);

    const status = getSubmissionStatus(submission);
    const deadlines = useSelector<AppState, Deadlines>(state => getEditionDeadlines(state.edition as Edition)); // edition cannot be null at this point

    const { sent, declined, comment } = submission;

    return <Step { ...props }
                 label={ t('steps.plan.header') }
                 active={ true } completed={ sent } declined={ declined } waiting={ status == "awaiting" }
                 until={ deadlines.proposal }
                 state={ <Status submission={ submission } /> }>
        <p>{ t(`steps.plan.info.${ status }`) }</p>

        { comment && <Box pb={ 2 }><PlanComment/></Box> }

        <PlanActions/>
    </Step>;
}
